using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.IO;

namespace GrBoard
{
	[Serializable]
	/// <summary>
	/// Article ̊Tv̐łB
	/// </summary>
	public class Article : IDisposable, IComparable
	{
		const int FRAGMENT_SIZE = 200;

		private DateTime date;
		private string subject;
		private string author;
		private string[] content;
		private Image picture;

		static private ArticleComparer comparer;

		#region ArticleComparer NX

		class ArticleComparer : IComparer
		{
			#region IComparer o

			public int Compare(object x, object y)
			{
				// TODO:  Comparer.Compare ǉ܂B
				if (x == null)
				{
					if (y == null)
					{
						return 0;
					}
					CheckType(y);
					return -1;
				}
				else if (y == null)
				{
					CheckType(x);
					return 1;
				}
				long x1 = CheckType(x);
				long y1 = CheckType(y);
				long l = x1 - y1;
				if (l < 0) 
				{
					return -1;
				}
				else if (l > 0)
				{
					return 1;
				}
				return 0;
			}

			#endregion

			private long CheckType(Object o)
			{
				if (o is long)
				{
					return (long)o;
				}
				if (!(o is Article))
				{
					throw new ArgumentException("bad type:" + o.GetType().ToString());
				}
				return ((Article)o).date.Ticks;
			}
		}

		#endregion

		public Article()
		{
			// 
			// TODO: RXgN^ WbNɒǉĂB
			//
			date = DateTime.Now;
		}

		// {A2iKRXgNV́AOT|[gĂ錾ł͖Ӗɋ߂B
		// ̏ꍇA炩DateTimeɂID擾Ɏs邽߂ɁÂ悤ɍ쐬ĂB
		public void Setup(Contribution c)
		{
			subject = c.Subject;
			author = c.Author;
			ArrayList a = new ArrayList();
			picture = new Bitmap(c.Width, c.Height);
			using (Graphics g = Graphics.FromImage(picture))
			{
				g.Clear(Color.White);
				foreach (Contribution.ImageElement e in c.GetEnumerable())
				{
					e.Draw(g);
					string[] t = e.Texts;
					if (t != null)
					{
						foreach (string s in t)
						{
							a.Add(s);
						}
					}
				}
			}
			content = new string[a.Count];
			a.CopyTo(content);
		}

		public long Id
		{
			get
			{
				return date.Ticks;
			}
		}

		public string Url
		{
			get
			{
				return "ShowImage.aspx?article=" + date.Ticks.ToString();
			}
		}

		public string ThumbnailUrl
		{
			get
			{
				return "ShowImage.aspx?article=" + date.Ticks.ToString() + "&type=thumbnail";
			}
		}

		public string Fragment
		{
			get
			{
				StringBuilder sb = new StringBuilder();
				foreach (string s in content)
				{
					sb.Append(s).Append('\n');
					if (sb.Length > FRAGMENT_SIZE)
					{
						break;
					}
				}
				if (sb.Length > 1 && sb[sb.Length - 1] == '\n')
				{
					sb.Length = sb.Length - 1;
				}
				return sb.ToString();
			}
		}

		public void Write(Stream stm)
		{
			try
			{
				picture.Save(stm, ImageFormat.Png);
			}
			catch (Exception e)
			{
				String s = e.Message;
			}
		}

		// Dummy
		public bool ThumbnailCallback()
		{
			return false;
		}

		public void WriteThumbnail(Stream stm)
		{
			Image.GetThumbnailImageAbort callback =
				new Image.GetThumbnailImageAbort(ThumbnailCallback);
			int w = picture.Width / 10;
			int h = picture.Height / 10;
			try
			{
				Image thum = picture.GetThumbnailImage(w, h, callback, IntPtr.Zero);
				thum.Save(stm, ImageFormat.Png);
			}
			catch
			{
			}
		}

		public static IComparer Comparer
		{
			get
			{
				if (comparer == null)
				{
					comparer = new ArticleComparer();
				}
				return comparer;
			}
		}

		/// <summary>
		/// ̋Lꂽtw肳ꂽLꂽtƓ𔻒肷B
		/// </summary>
		/// <param name="oponent">rL</param>
		/// <returns></returns>
		public bool IsSameDate(Article oponent)
		{
			return (date.Year == oponent.date.Year 
				&& date.DayOfYear == oponent.date.DayOfYear);
		}

		/// <summary>
		/// ̋Lꂽtw肳ꂽtƓ𔻒肷B
		/// </summary>
		/// <param name="dt">rt</param>
		/// <returns></returns>
		public bool IsSameDate(DateTime dt)
		{
			return (date.Year == dt.Year 
				&& date.DayOfYear == dt.DayOfYear);
		}

		/// <summary>
		/// ̋Lꂽt(܂)
		/// </summary>
		public DateTime Date
		{
			get
			{
				return date;
			}
			set
			{
				date = value;
			}
		}

		/// <summary>
		/// ̋L̎(^Cg)
		/// </summary>
		public string Subject
		{
			get
			{
				return subject;
			}
			set
			{
				subject = value;
			}
		}

		/// <summary>
		/// ̋L̍쐬
		/// </summary>
		public string Author
		{
			get 
			{
				return author;
			}
			set
			{
				author = value;
			}
		}

		/// <summary>
		/// ̋L̖{
		/// </summary>
		public string[] Content
		{
			get
			{
				return content;
			}
			set
			{
				content = value;
			}
		}

		/// <summary>
		/// ̋LɕtꂽG
		/// </summary>
		public Image Picture
		{
			get
			{
				return picture;
			}
			set
			{
				picture = value;
			}
		}
		#region IDisposable o

		public void Dispose()
		{
			// TODO:  Article.Dispose ǉ܂B
			if (picture != null)
			{
				picture.Dispose();
				picture = null;
			}
		}

		#endregion

		#region IComparable o

		public int CompareTo(object obj)
		{
			// TODO:  Article.CompareTo ǉ܂B
			if (obj == null)
			{
				return 1;
			}
			if (!(obj is Article))
			{
				throw new ArgumentException("bad type:" + obj.GetType().ToString());
			}

			long l = date.Ticks - ((Article)obj).date.Ticks;
			if (l < 0) 
			{
				return -1;
			}
			else if (l > 0)
			{
				return 1;
			}
			return 0;
		}

		#endregion
	}
}
