using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Xml.Serialization;

namespace GrBoard
{
	[Serializable]
	[XmlRoot]
	/// <summary>
	/// NCAgT[o[֓eɎgp钊ۓIȃC[WBB
	/// </summary>
	public class Contribution
	{
		public Contribution()
		{
			// 
			// TODO: RXgN^ WbNɒǉĂB
			//
			elements = new ArrayList();
		}

		private string subject;
		private string author;
		private ArrayList elements;
		private int width;
		private int height;

		internal IEnumerable GetEnumerable()
		{
			// TODO:  Contribution.GetEnumerator ǉ܂B
			return elements;
		}

		public int Width
		{
			get
			{
				return width;
			}
			set
			{
				width = value;
			}
		}

		public int Height
		{
			get
			{
				return height;
			}
			set
			{
				height = value;
			}
		}

		public string Subject
		{
			get
			{
				return subject;
			}
			set
			{
				subject = value;
			}
		}

		public string Author
		{
			get
			{
				return author;
			}
			set
			{
				author = value;
			}
		}


		public ImageElement[] Elements
		{
			get 
			{
				ImageElement[] a = new ImageElement[elements.Count];
				elements.CopyTo(a);
				return a;
			}
			set
			{
				elements = new ArrayList(value);
			}
		}

		internal void Add(ImageElement elem)
		{
			Debug.Assert(elements != null, "");
			Debug.Assert(elem != null, "nullQ");

			elements.Add(elem);
		}

		/// <summary>
		/// `ǉB
		/// </summary>
		/// <param name="pnts">e̊Jn_</param>
		public void Add(ICollection pnts)
		{
			Debug.Assert(elements != null, "");
			Debug.Assert(pnts != null, "nullQ");

			elements.Add(new Line(pnts));
		}

		/// <summary>
		/// eLXgǉB
		/// </summary>
		/// <param name="pnts">e̊Jn_</param>
		/// <param name="txts">̃Xg</param>
		/// <param name="fnt">`ɎgptHg</param>
		public void Add(ICollection pnts, ICollection txts, Font fnt)
		{
			Debug.Assert(elements != null, "");
			Debug.Assert(pnts != null && txts != null && fnt != null, "nullQ");
			Debug.Assert(pnts.Count == txts.Count, "vfs");

			elements.Add(new Text(pnts, txts, fnt));
		}

		[Serializable]
		[XmlInclude(typeof(Line)), XmlInclude(typeof(Text))]
		abstract public class ImageElement
		{
			protected PointF[] points;
			protected Color color;

			public ImageElement()
			{
			}

			protected ImageElement(ICollection pnts)
			{
				points = new PointF[pnts.Count];
				pnts.CopyTo(points, 0);
			}

			public PointF[] Points
			{
				get
				{
					return points;
				}
				set
				{
					points = value;
				}
			}

			public int ColorArgb
			{
				get
				{
					return color.ToArgb();
				}
				set
				{
					color = Color.FromArgb(value);
				}
			}

			virtual public string[] Texts
			{
				get
				{
					return null;
				}
				set
				{
					throw new NotSupportedException("Texts setter is not supported");
				}
			}

			abstract internal void Draw(Graphics g);
		}

		[Serializable]
		public class Line : ImageElement
		{
			public Line()
			{
			}

			internal Line(ICollection pnts)
				: base(pnts)
			{
			}

			override internal void Draw(Graphics g)
			{
				Debug.Assert(points != null, "no points");

				using (Pen pen = new Pen(color))
				{
					g.DrawLines(pen, points);
				}
			}
		}

		[Serializable]
		public class Text : ImageElement
		{
			protected string[] texts;
			protected Font font;

			internal Text(ICollection pnts, ICollection txts, Font fnt)
				: base(pnts)
			{
				texts = new string[txts.Count];
				txts.CopyTo(texts, 0);
				font = fnt;
			}

			public Text()
			{
			}

			public FontValue Font
			{
				get
				{
					FontValue f = new FontValue(font);
					return f;
				}
				set
				{
					font = value.ToFont();
				}
			}

			override public string[] Texts
			{
				get
				{
					return texts;
				}
				set
				{
					texts = value;
				}
			}

			override internal void Draw(Graphics g)
			{
				Debug.Assert(points != null, "no points");
				Debug.Assert(font != null, "no font");
				Debug.Assert(texts != null, "no text");
				
				using (Brush b = new SolidBrush(color))
				{
					for (int i = 0; i < texts.Length; i++)
					{
						g.DrawString(texts[i], font, b, points[i]);
					}
				}
			}
		}

		[Serializable]
		public class FontValue
		{
			public FontValue()
			{
			}
			internal FontValue(Font f)
			{
				FontFamily = f.FontFamily.Name;
				GdiCharSet = f.GdiCharSet;
				GdiVerticalFont = f.GdiVerticalFont;
				Height = f.Height;
				Name = f.Name;
				Size = f.Size;
				Style = (int)f.Style;
				Unit = (int)f.Unit;
			}

			internal Font ToFont()
			{
				Font font = new Font(FontFamily, Size, (FontStyle)Style, (GraphicsUnit)Unit, GdiCharSet, GdiVerticalFont);
				return font;
			}

			public string FontFamily;
			public byte GdiCharSet;
			public bool GdiVerticalFont;
			public int Height;
			public string Name;
			public float Size; 
			public int Style; 
			public int Unit;
		}
	}
}
